package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.birthday.UserBirthday;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;


/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:view.sub")
@CommandInfo(name = "view", description = "Add your birthday to this guild")
public class ViewSubCommand extends BatSubCommand {
    @Autowired
    public ViewSubCommand(GuildService guildService) {
        super.addOption(OptionType.USER, "user", "The user to view the birthday of", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        BirthdayProfile profile = guild.getProfile(BirthdayProfile.class);

        if (!profile.hasChannelSetup()) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Birthdays have not been enabled in this guild. Please ask an administrator to enable them.")
                    .build()).queue();
            return;
        }

        OptionMapping birthdayOption = interaction.getOption("user");
        if (birthdayOption == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You must provide a birthday")
                    .build()).queue();
            return;
        }

        Member targetMember = birthdayOption.getAsMember();
        if (targetMember == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You must provide a valid user")
                    .build()).queue();
            return;
        }
        UserBirthday birthday = profile.getBirthday(targetMember.getId());
        if (birthday == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("This user does not have a birthday set")
                    .build()).queue();
            return;
        }
        if (birthday.isHidden() && !user.getId().equals(targetMember.getId())) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s has their birthday set to private".formatted(targetMember.getAsMention()))
                    .build()).queue();
            return;
        }

        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("%s's birthday is <t:%s>".formatted(member.getAsMention(), birthday.getBirthday().toInstant().toEpochMilli()/1000))
                .build()).queue();
    }
}
